#include "Gamehand.h"

Gamehand::Gamehand(PhysxScene* jenga, ref_ptr<Group> osgScene, PxU32 filterGroup, PxU32 filterMask)
: _filterGroup(filterGroup)
, _filterMask(filterMask)
{
	physxScene = jenga;
	physxScene->PX::createMaterial("tip", 10.0, 10.0, 0.f);
	Gamehand::osgScene = new osg::Group;

	ShaderProgram* handShader = new ShaderProgram(Gamehand::osgScene);

	handShader->setVertexShader("data/Jenga/handShader.vert");
	handShader->getVertexShader()->addCodeFileInjection("data/phong_vert.glsl", "//HOTSPOT_PRE_MAIN");
	handShader->getVertexShader()->addCodeTextInjection("phong();", "//HOTSPOT_START_MAIN");



	handShader->setFragmentShader("data/Jenga/handShader.frag");
	handShader->getFragmentShader()->addCodeFileInjection("data/phong_frag.glsl", "//HOTSPOT_PRE_MAIN");

	osgScene->addChild(Gamehand::osgScene);

	float hand_w = 3.5f;
	float finger_padding = 0.2f;
	buildHand(hand_w, finger_padding);
}

void Gamehand::buildHandBone(enum Gamehand::objects type, float len_x, float len_y, float len_z, float trans_x, float trans_y, float trans_z, float angle, osg::Vec3 axis) {
	osg::ref_ptr<osg::Geode> metacarpal = new osg::Geode;
	osg::ref_ptr<osg::ShapeDrawable> metacarpalShape = new osg::ShapeDrawable(new osg::Box(osg::Vec3(), len_x, len_y, len_z));

	metacarpalShape->setColor(osg::Vec4f(0, 1, 1, 1));
	metacarpal->addDrawable(metacarpalShape);
	osg::ref_ptr<osg::MatrixTransform> metacarpalTransform = new osg::MatrixTransform(osg::Matrix::rotate(angle, axis) * osg::Matrix::translate(trans_x, trans_y, trans_z));
	metacarpalTransform->addChild(metacarpal.get());
	osgScene->addChild(metacarpalTransform.get());
	physxScene->PX::createBox(type, osg::Vec3f(len_x, len_y, len_z), 1.0, "standard", PX::ALL, _filterGroup, _filterMask);
	physxScene->PX::setMatrix(type, metacarpalTransform->getMatrix());
	transforms[type] = metacarpalTransform.get();
}

void Gamehand::buildHandSphere(enum Gamehand::objects type, float radius, float trans_x, float trans_y, float trans_z, float angle, osg::Vec3 axis) {
	osg::ref_ptr<osg::Geode> metacarpal = new osg::Geode;
	osg::ref_ptr<osg::ShapeDrawable> metacarpalShape = new osg::ShapeDrawable(new osg::Sphere(osg::Vec3(), radius));

	metacarpalShape->setColor(osg::Vec4f(0, 1, 1, 1));
	metacarpal->addDrawable(metacarpalShape);
	osg::ref_ptr<osg::MatrixTransform> metacarpalTransform = new osg::MatrixTransform(osg::Matrix::rotate(angle, axis) * osg::Matrix::translate(trans_x, trans_y, trans_z));
	metacarpalTransform->addChild(metacarpal.get());
	osgScene->addChild(metacarpalTransform.get());
	physxScene->PX::createSphere(type, radius, 1.0, "tip", PX::ALL, _filterGroup, _filterMask);
	physxScene->PX::setMatrix(type, metacarpalTransform->getMatrix());
	transforms[type] = metacarpalTransform.get();
}

void Gamehand::buildHand(float hand_w, float finger_padding) {
	float finger_w = (hand_w - 3 * finger_padding) / 4;

	// Add Hand

	// Add Palm
	buildHandBone(PALM, hand_w, 3, 0.8f, 0, -8, -6, 0, osg::Vec3(0, 0, 1));
	
	// connect palm to world for debugging reasons
	//int jID = physxScene->getNextJointId();
	//physxScene->PX::createFixedJoint(jID, PALM, osg::Matrix::translate(osg::Vec3(0, 0, 0)), WORLD, osg::Matrix::translate(osg::Vec3(0, -8, -6)));


	// Add thumb

	// metacarpal
	buildHandBone(METACARPAL, finger_w + 0.1, 1.2, 0.6, -hand_w / 2 - 0.65, -8.5, -6, PI_4, osg::Vec3(0, 0, 1));

	// revolute joint palm - metacarpal
	int jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, PALM, osg::Matrix::rotate(PI_2, osg::Vec3(0, 0, 1)) * osg::Matrix::translate(osg::Vec3(-hand_w / 2, -1.25, 0)), METACARPAL, osg::Matrix::rotate(PI_4, osg::Vec3(0, 0, 1)) * osg::Matrix::translate(osg::Vec3(0, -0.7, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, PI_4, 0);

	// proximal
	buildHandBone(P_PHALANGE0, finger_w + 0.1, 1.6, 0.6, -hand_w / 2 - 1.6, -6.8, -6, 0, osg::Vec3(0, 0, 1));

	// revolute joint metacarpal - proximal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, METACARPAL, osg::Matrix::rotate(-PI_2, osg::Vec3(0, 1, 0)) * osg::Matrix::translate(osg::Vec3(0, 0.7, 0)), P_PHALANGE0, osg::Matrix::translate(osg::Vec3(0, -0.8, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2 - PI_4,  PI_4 / 2, 0);

	// distal
	buildHandSphere(S_PHALANGE0, 0.3, -hand_w / 2 - 1.6, -5.5, -6, 0, osg::Vec3(0, 0, 1));

	// revolute joint proximal - distal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, P_PHALANGE0, osg::Matrix::translate(osg::Vec3(0, 0.8, 0)), S_PHALANGE0, osg::Matrix::translate(osg::Vec3(0, -0.7, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);


	// Add forefinger

	// proximal
	buildHandBone(P_PHALANGE1, finger_w, 1.2, 0.6, -(hand_w / 2) + (finger_w / 2), -5.9, -6, PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint palm - proximal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, PALM, osg::Matrix::rotate(PI_4 / 4, osg::Vec3(0, 0, 1)) * osg::Matrix::translate(osg::Vec3(-(hand_w / 2) + (finger_w / 2), 1.5, 0)), P_PHALANGE1, osg::Matrix::translate(osg::Vec3(0, -0.6, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, PI_4, 0);

	// middle
	buildHandBone(M_PHALANGE1, finger_w, 1.0, 0.6, -(hand_w / 2) + (finger_w / 2), -4.8, -6, PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint proximal - middle
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, P_PHALANGE1, osg::Matrix::translate(osg::Vec3(0, 0.6, 0)), M_PHALANGE1, osg::Matrix::translate(osg::Vec3(0, -0.5, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);

	// distal
	buildHandSphere(S_PHALANGE1, 0.3, -(hand_w / 2) + (finger_w / 2), -3.95, -6, PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint proximal - middle
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, M_PHALANGE1, osg::Matrix::translate(osg::Vec3(0, 0.5, 0)), S_PHALANGE1, osg::Matrix::translate(osg::Vec3(0, -0.5, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);


	// Add middle finger

	// proximal
	buildHandBone(P_PHALANGE2, finger_w, 1.4, 0.6, -(hand_w / 2) + 1.5 * finger_w + finger_padding, -5.8, -6, 0, osg::Vec3(0, 0, 1));

	// revolute joint palm - proximal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, PALM, osg::Matrix::translate(osg::Vec3(-(hand_w / 2) + 1.5 * finger_w + finger_padding, 1.5, 0)), P_PHALANGE2, osg::Matrix::translate(osg::Vec3(0, -0.7, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, PI_4, 0);

	// middle
	buildHandBone(M_PHALANGE2, finger_w, 1.2, 0.6, -(hand_w / 2) + 1.5 * finger_w + finger_padding, -4.5, -6, 0, osg::Vec3(0, 0, 1));

	// revolute joint proximal - middle
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, P_PHALANGE2, osg::Matrix::translate(osg::Vec3(0, 0.7, 0)), M_PHALANGE2, osg::Matrix::translate(osg::Vec3(0, -0.5, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);

	// distal
	buildHandSphere(S_PHALANGE2, 0.3, -(hand_w / 2) + 1.5 * finger_w + finger_padding, -3.5, -6, 0, osg::Vec3(0, 0, 1));

	// revolute joint middle - distal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, M_PHALANGE2, osg::Matrix::translate(osg::Vec3(0, 0.6, 0)), S_PHALANGE2, osg::Matrix::translate(osg::Vec3(0, -0.6, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);


	// Add ring finger

	// proximal
	buildHandBone(P_PHALANGE3, finger_w, 1.2, 0.6, hand_w / 2 - 1.5 * finger_w - finger_padding, -5.9, -6, -PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint palm - proximal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, PALM, osg::Matrix::rotate(-PI_4 / 4, osg::Vec3(0, 0, 1)) * osg::Matrix::translate(osg::Vec3(hand_w / 2 - 1.5 * finger_w - finger_padding, 1.5, 0)), P_PHALANGE3, osg::Matrix::translate(osg::Vec3(0, -0.6, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, PI_4, 0);

	// middle
	buildHandBone(M_PHALANGE3, finger_w, 1.1, 0.6, hand_w / 2 - 1.5 * finger_w - finger_padding, -4.75, -6, -PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint proximal - middle
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, P_PHALANGE3, osg::Matrix::translate(osg::Vec3(0, 0.6, 0)), M_PHALANGE3, osg::Matrix::translate(osg::Vec3(0, -0.55, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);

	// distal
	buildHandSphere(S_PHALANGE3, 0.3, hand_w / 2 - 1.5 * finger_w - finger_padding, -3.85, -6, -PI_4 / 4, osg::Vec3(0, 0, 1));

	// revolute joint middle - distal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, M_PHALANGE3, osg::Matrix::translate(osg::Vec3(0, 0.55, 0)), S_PHALANGE3, osg::Matrix::translate(osg::Vec3(0, -0.5, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);


	// Add little finger

	// proximal
	buildHandBone(P_PHALANGE4, finger_w, 0.8, 0.5, hand_w / 2 - finger_w / 2, -6.1, -6, -PI_4 / 2, osg::Vec3(0, 0, 1));

	// revolute joint palm - proximal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, PALM, osg::Matrix::rotate(-PI_4 / 2, osg::Vec3(0, 0, 1)) * osg::Matrix::translate(osg::Vec3(hand_w / 2 - finger_w / 2, 1.5, 0)), P_PHALANGE4, osg::Matrix::translate(osg::Vec3(0, -0.4, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, PI_4, 0);

	// middle
	buildHandBone(M_PHALANGE4, finger_w, 0.7, 0.5, hand_w / 2 - finger_w / 2, -5.35, -6, -PI_4 / 2, osg::Vec3(0, 0, 1));

	// revolute joint proximal - middle
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, P_PHALANGE4, osg::Matrix::translate(osg::Vec3(0, 0.4, 0)), M_PHALANGE4, osg::Matrix::translate(osg::Vec3(0, -0.35, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);

	// distal
	buildHandSphere(S_PHALANGE4, 0.25, hand_w / 2 - finger_w / 2, -4.7, -6, -PI_4 / 2, osg::Vec3(0, 0, 1));

	// revolute joint middle - distal
	jID = physxScene->getNextJointId();
	physxScene->PX::createRevoluteJoint(jID, M_PHALANGE4, osg::Matrix::translate(osg::Vec3(0, 0.35, 0)), S_PHALANGE4, osg::Matrix::translate(osg::Vec3(0, -0.4, 0)));
	physxScene->PX::setRevoluteJointLimit(jID, -PI_2, 0, 0);
}

bool Gamehand::fingerIdIsUnassigned(int32_t id) {
	if (id != NULL && finger_id[0] != id && finger_id[1] != id && finger_id[2] != id && finger_id[3] != id && finger_id[4] != id)
		return true;
	else
		return false;
}

int32_t Gamehand::getLeftmostUnassignedFingerID(Leap::FingerList fingers) {
	Leap::Finger leftmost;
	bool found = false;
	for (int i = 0; i < fingers.count(); i++)
	{
		if (!(fingers[i].isValid() && fingerIdIsUnassigned(fingers[i].id())))
			continue;
		if (!found) {
			leftmost = fingers[i];
			found = true;
		}
		else if (fingers[i].stabilizedTipPosition().x < leftmost.stabilizedTipPosition().x) {
			found = true;
			leftmost = fingers[i];
		}
	}
	if (!found)
		return NULL;
	else
		return leftmost.id();
}

void Gamehand::synchronizeHand(Leap::Frame frame, float angle, bool init) {
	Leap::FingerList fingerList = frame.fingers();

	if (frame.hands().count() > 0)
		synchronizePalm(PALM, frame.hands()[0], angle);

	// Initialize finger_ids
	if (init)
	for (int i = 0; i < 5; i++)
		finger_id[i] = NULL;

	// Assign fingers to IDs if all fingers were lost
	if (fingerList.count() == 5 && finger_id[0] == NULL && finger_id[1] == NULL && finger_id[2] == NULL && finger_id[3] == NULL && finger_id[4] == NULL)
	{
		for (int i = 0; i < fingerList.count(); i++)
			finger_id[i] = getLeftmostUnassignedFingerID(fingerList);
		init = false;
	}

	if (!init)
	{
		enum Gamehand::objects physx_fingers[] = { S_PHALANGE0, S_PHALANGE1, S_PHALANGE2, S_PHALANGE3, S_PHALANGE4 };
		// Synchronize fingers with Leap fingers
		for (int i = 0; i < 5; i++) {
			if (finger_id[i] != NULL && Leap::Finger(frame.pointable(finger_id[i])).isValid())
				synchronizeFinger(physx_fingers[i], Leap::Finger(frame.pointable(finger_id[i])), angle);
			else {
				finger_id[i] = NULL;
				finger_id[i] = getLeftmostUnassignedFingerID(fingerList); // To be replace by something smarter
				if (finger_id[i] != NULL) {
					// Retrack the finger
					synchronizeFinger(physx_fingers[i], Leap::Finger(frame.pointable(finger_id[i])), angle);
				}
			}
		}

	}
}

void Gamehand::synchronizePalm(int id, const Leap::Hand& hand, float angle) {
	osg::Matrix translationMatrix = osg::Matrix::identity();
	translationMatrix.setTrans(toOsgVec3(hand.palmPosition()) / 20.f - osg::Vec3(0.0, 0.0, 15.0));

	osg::Matrix rotationMatrix = osg::Matrix::identity();
	rotationMatrix.makeRotate(osg::PI - angle, Vec3(0.0, 0.0, 1.0));

	// target vector has to be translated due to camera rotation
	osg::Vec3f diff = (translationMatrix * rotationMatrix).getTrans() - transforms[id]->getMatrix().getTrans();
	physxScene->PX::setLinearVelocity(id, diff * 20.0f);

	osg::Vec3 rotation = getRotationInDirection(*transforms[id], osg::Vec3(0.0, 0.0, -1.0), osg::Matrix::inverse(rotationMatrix) * toOsgVec3(hand.palmNormal()));
	osg::Vec3 rotation2 = getRotationInDirection(*transforms[id], osg::Vec3(0.0, 1.0, 0.0), osg::Matrix::inverse(rotationMatrix) * toOsgVec3(hand.direction()));
	physxScene->PX::setAngularVelocity(id, (rotation + rotation2) * 40.0f);
}


void Gamehand::synchronizeFinger(int id, const Leap::Finger& finger, float angle) {
	osg::Matrix translationMatrix = osg::Matrix::identity();
	translationMatrix.setTrans(toOsgVec3(finger.tipPosition()) / 20.f - osg::Vec3(0.0, 0.0, 15.0));

	osg::Matrix rotationMatrix = osg::Matrix::identity();
	rotationMatrix.makeRotate(osg::PI - angle, Vec3(0.0, 0.0, 1.0));

	// target vector has to be translated due to camera rotation
	osg::Vec3f diff = (translationMatrix * rotationMatrix).getTrans() - transforms[id]->getMatrix().getTrans();
	physxScene->PX::setLinearVelocity(id, diff * 20.0f);

	// target rotation has to be rotated by the inverse rotation matrix
	osg::Vec3 rotation = getRotationInDirection(*transforms[id], osg::Vec3(0.0, 1.0, 0.0), osg::Matrix::inverse(rotationMatrix) * toOsgVec3(finger.direction()));
	physxScene->PX::setAngularVelocity(id, rotation * 40.0f);
}