#pragma once

#include <osg/Referenced>
#include <osg/ref_ptr>
#include <osg/Geode>
#include <osg/ShapeDrawable>
#include <osg/Geometry>
#include <osg/MatrixTransform>

#include "../TeetairApplication.h"

#include <iostream>

using namespace osg;
class Jenga;

class Gamehand : public osg::Referenced
{
private:
	enum objects{
		WORLD = 0,
		GROUND = 1,
		FINGER0 = 2,
		FINGER1 = 3,
		PALM = 4,
		METACARPAL = 5,
		P_PHALANGE0 = 6,
		D_PHALANGE0 = 7,
		P_PHALANGE1 = 8,
		M_PHALANGE1 = 9,
		D_PHALANGE1 = 10,
		P_PHALANGE2 = 11,
		M_PHALANGE2 = 12,
		D_PHALANGE2 = 13,
		P_PHALANGE3 = 14,
		M_PHALANGE3 = 15,
		D_PHALANGE3 = 16,
		P_PHALANGE4 = 17,
		M_PHALANGE4 = 18,
		D_PHALANGE4 = 19,
		S_PHALANGE0 = 20,
		S_PHALANGE1 = 21,
		S_PHALANGE2 = 22,
		S_PHALANGE3 = 23,
		S_PHALANGE4 = 24,
		BRICK_START = 25
	};
	int32_t finger_id[5];

public:
	Gamehand(PhysxScene* jenga, ref_ptr<Group> osgScene, PxU32 filterGroup, PxU32 filterMask);
	virtual void synchronizeHand(Leap::Frame frame, float angle, bool init);
	typedef unordered_map<int, ref_ptr<osg::MatrixTransform>> TransformMap;
	TransformMap transforms;

protected:

private:
	virtual bool fingerIdIsUnassigned(int32_t id);
	virtual int32_t getLeftmostUnassignedFingerID(Leap::FingerList fingers);
	virtual void synchronizePalm(int id, const Leap::Hand& hand, float angle);
	virtual void synchronizeFinger(int id, const Leap::Finger& finger, float angle);
	virtual void buildHand(float hand_w, float finger_padding);
	virtual void buildHandBone(enum Gamehand::objects type, float len_x, float len_y, float len_z, float trans_x, float trans_y, float trans_z, float angle, osg::Vec3 axis);
	virtual void buildHandSphere(enum Gamehand::objects type, float radius, float trans_x, float trans_y, float trans_z, float angle, osg::Vec3 axis);
	ref_ptr<Group> osgScene;
	PhysxScene* physxScene;
	PxU32 _filterGroup;
	PxU32 _filterMask;
};

