#include "Ball.h"
#include "Teeter.h"
#include "Level.h"
#include "Ribbon.h"
#include "Board.h"
#include "Ground.h"
#include "Block.h"
#include "BlockCheckpoint.h"

Ball::Ball(Level& level)
: GameObject(level)
{
	ball = new osg::Geode;
	osg::ref_ptr<osg::ShapeDrawable> ballShape = new osg::ShapeDrawable(new osg::Sphere(osg::Vec3(), 0.3));
	ballShape->setColor(osg::Vec4(0.9, 0.1, 0.1, 1.0));
	ball->addDrawable(ballShape.get());
	ballTransform = new osg::MatrixTransform(osg::Matrix::identity());
	ballTransform->addChild(ball.get());
	getLevel().addChild(ballTransform.get());
	getTeeter().setTransform(getId(), ballTransform.get());

	shader = new ShaderProgram(ballTransform);
	shader->setVertexShader("data/Teeter/standardShader.vert");
	shader->getVertexShader()->addCodeFileInjection("data/phong_vert-withgeom.glsl", "//HOTSPOT_PRE_MAIN");
	shader->getVertexShader()->addCodeTextInjection("phong();", "//HOTSPOT_START_MAIN");
	shader->setGeometryShader("data/Teeter/portalBallShader.geom");
	shader->setFragmentShader("data/Teeter/portalBallShader.frag");
	shader->getFragmentShader()->addCodeFileInjection("data/phong_frag.glsl", "//HOTSPOT_PRE_MAIN");

	ballTransform->getOrCreateStateSet()->addUniform(new Uniform("a_PortalActive", false));

	// add the ribbon
	ribbon = new Ribbon(Vec3(0.5f,0.5f,1.0f), ballTransform.get(), getId(), getTeeter(), getLevel().getBoard()->getTransform());

	// add ball to PhysX
	getTeeter().PX::createSphere(getId(), 0.3, 1, "holz", PX::ALL, Level::Groups::BALL, Level::Groups::GROUND | Level::Groups::BOARD);
	getTeeter().PX::setEnableCCD(getId(), true);
	getTeeter().PX::setMatrix(getId(), ballTransform->getMatrix());

	reset();
}

void Ball::reset() {
	ref_ptr<BlockCheckpoint> cp = getLevel().getLastCheckpoint();
	if (cp) {
		Vec3 resetPosition = Vec3(cp->getX(), cp->getY(), cp->getHeight() + 0.5);
		getTeeter().PX::setMatrix(getId(), osg::Matrix::translate(resetPosition));
		getTeeter().PX::clearForce(getId());
		getTeeter().PX::setLinearVelocity(getId(), Vec3());
		getTeeter().PX::setAngularVelocity(getId(), Vec3());
	}
}

void Ball::contact(Ground& ground) {
	ground.contactedBy(*this);
}

void Ball::contact(Block& block) {
	block.contactedBy(*this);
}

void Ball::trigger(Block& block) {
	block.triggeredBy(*this);
}

void Ball::untrigger(Block& block) {
	block.untriggeredBy(*this);
}

Ball::~Ball() {
	ballTransform->removeUpdateCallback(ribbon.get());
}