#include "Block.h"
#include "Teeter.h"
#include "Level.h"
#include "Board.h"
#include "Ball.h"

Block::Block(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group, bool standalone, std::string type)
: GameObject(level)
, x(x)
, y(y)
, group(group)
{
	buildBlock(b, standalone, type);
}

void Block::buildBlock(int brightness, bool standalone, std::string type) {
	int startHeight = 0;
	int endHeight = 0;
	int b = -1;

	if (brightness < 57) {
		return; // Hole or wrong encoding
	}
	else if (brightness < 73) {
		startHeight = 0;
		b = brightness - 57;
	}
	else if (brightness == 73) {
		startHeight = 1;
		endHeight = 1;
	}
	else if (brightness < 86) {
		startHeight = 1;
		b = brightness - 74;
	}
	else if (brightness == 86) {
		startHeight = 2;
		endHeight = 2;
	}
	else if (brightness < 95) {
		startHeight = 2;
		b = brightness - 87;
	}
	else if (brightness == 95) {
		startHeight = 3;
		endHeight = 3;
	}
	else if (brightness < 100) {
		startHeight = 3;
		b = brightness - 96;
	}
	else if (brightness == 100) {
		startHeight = 4;
		endHeight = 4;
	}

	if (b != -1) {
		endHeight = b / 4 + startHeight + 1;
		GradientDirection dir = static_cast<GradientDirection>(static_cast<int>(GradientDirection::TOP) + (b % 4));
		height = endHeight / 2.0f;
		buildConvexBlock(startHeight / 2.0f, endHeight / 2.0f, dir, standalone, type);
	}
	else {
		height = startHeight / 2.0f;
		buildHeightBlock(startHeight / 2.0f, standalone, type);
	}
}

void Block::buildConvexBlock(float startHeight, float endHeight, GradientDirection dir, bool standalone, std::string type) {
	blockStartHeight = startHeight;
	blockEndHeight = endHeight;
	blockDirection = dir;

	float bl = (dir == GradientDirection::TOP || dir == GradientDirection::RIGHT ? endHeight : startHeight);
	float br = (dir == GradientDirection::TOP || dir == GradientDirection::LEFT ? endHeight : startHeight);
	float tl = (dir == GradientDirection::BOTTOM || dir == GradientDirection::RIGHT ? endHeight : startHeight);
	float tr = (dir == GradientDirection::BOTTOM || dir == GradientDirection::LEFT ? endHeight : startHeight);

	ref_ptr<Vec3Array> vertices = new Vec3Array;
	vertices->push_back(Vec3(-0.5, -0.5, 0.0));	// bl bottom	(0)
	vertices->push_back(Vec3(-0.5, -0.5, bl));	// bl top		(1)
	vertices->push_back(Vec3(-0.5, 0.5, 0.0));	// tl bottom	(2)
	vertices->push_back(Vec3(-0.5, 0.5, tl));	// tl top		(3)
	vertices->push_back(Vec3(0.5, -0.5, 0.0));	// br bottom	(4)
	vertices->push_back(Vec3(0.5, -0.5, br));	// br top		(5)
	vertices->push_back(Vec3(0.5, 0.5, 0.0));	// tr bottom	(6)
	vertices->push_back(Vec3(0.5, 0.5, tr));	// tr top		(7)

	if (standalone) {
		for (Vec3Array::iterator i = vertices->begin(); i != vertices->end(); ++i) {
			(*i) += Vec3(0, 0, -height / 2.0f);
		}
	}

	ref_ptr<osg::Geometry> geometry = new osg::Geometry();
	geometry->setVertexArray(vertices.get());


	ref_ptr<Vec3Array> normals = new Vec3Array;

	ref_ptr<osg::DrawElementsUInt> left = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	left->push_back(0);
	left->push_back(1);
	left->push_back(3);
	left->push_back(2);
	geometry->addPrimitiveSet(left.get());
	normals->push_back((vertices->at(0) - vertices->at(1)) ^ (vertices->at(0) - vertices->at(3)));

	ref_ptr<osg::DrawElementsUInt> top = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	top->push_back(2);
	top->push_back(3);
	top->push_back(7);
	top->push_back(6);
	geometry->addPrimitiveSet(top.get());
	normals->push_back((vertices->at(2) - vertices->at(3)) ^ (vertices->at(2) - vertices->at(7)));

	ref_ptr<osg::DrawElementsUInt> right = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	right->push_back(6);
	right->push_back(7);
	right->push_back(5);
	right->push_back(4);
	geometry->addPrimitiveSet(right.get());
	normals->push_back((vertices->at(6) - vertices->at(7)) ^ (vertices->at(6) - vertices->at(5)));

	ref_ptr<osg::DrawElementsUInt> bottom = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	bottom->push_back(4);
	bottom->push_back(5);
	bottom->push_back(1);
	bottom->push_back(0);
	geometry->addPrimitiveSet(bottom.get());
	normals->push_back((vertices->at(5) - vertices->at(4)) ^ (vertices->at(5) - vertices->at(1)));

	ref_ptr<osg::DrawElementsUInt> roof = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	roof->push_back(1);
	roof->push_back(5);
	roof->push_back(7);
	roof->push_back(3);
	geometry->addPrimitiveSet(roof.get());
	normals->push_back((vertices->at(1) - vertices->at(5)) ^ (vertices->at(1) - vertices->at(7)));

	ref_ptr<osg::DrawElementsUInt> basement = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS);
	basement->push_back(0);
	basement->push_back(2);
	basement->push_back(6);
	basement->push_back(4);
	geometry->addPrimitiveSet(basement.get());
	normals->push_back((vertices->at(0) - vertices->at(2)) ^ (vertices->at(0) - vertices->at(6)));

	geometry->setNormalArray(normals.get(), osg::Array::Binding::BIND_PER_PRIMITIVE_SET);

	if (standalone) {
		// create new geode
		ref_ptr<Geode> geode = new osg::Geode;
		geode->addDrawable(geometry);
		transform = new MatrixTransform(osg::Matrix::translate(x, y, height / 2.0f));
		getTeeter().setTransform(getId(), transform.get());
		transform->addChild(geode);
		getLevel().getBoard()->getShaderGroup(type)->addChild(transform);
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_TextureOffset", Vec3(x, y, height / 2.0f)));

		getTeeter().PX::createConvexBox(getId(), *vertices.get(), 1.0, "holz", PX::NO_GRAVITY, Level::Groups::BOARD, Level::Groups::BALL);
		getTeeter().PX::setMatrix(getId(), transform->getMatrix());
	}
	else {
		// add block to geode board
		for (Vec3Array::iterator i = vertices->begin(); i != vertices->end(); ++i) {
			*i._Ptr += Vec3(x, y, 0);
		}
		getLevel().getBoard()->addDrawable(geometry);

		getTeeter().PX::addConvexBoxToCompound(getLevel().getBoard()->getId(), *vertices.get(), "holz", Level::Groups::BOARD, Level::Groups::BALL);
		transform = getLevel().getBoard()->getTransform();
	}
}

void Block::buildHeightBlock(float height, bool standalone, std::string type) {
	blockStartHeight = height;
	blockEndHeight = height;
	blockDirection = GradientDirection::NONE;

	if (standalone) {
		// create new geode
		ref_ptr<Geode> geode = new osg::Geode;
		geode->addDrawable(new osg::ShapeDrawable(new osg::Box(osg::Vec3(), 1.0, 1.0, height)));
		transform = new MatrixTransform(osg::Matrix::translate(x, y, height / 2.0f));
		getTeeter().setTransform(getId(), transform.get());
		transform->addChild(geode);
		getLevel().getBoard()->getShaderGroup(type)->addChild(transform);
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_TextureOffset", Vec3(x, y, height / 2.0f)));

		getTeeter().PX::createBox(getId(), osg::Vec3(1.0, 1.0, height), 1.0f, "holz", PX::NO_GRAVITY, Level::Groups::BOARD, Level::Groups::BALL);
		getTeeter().PX::setMatrix(getId(), transform->getMatrix());
	}
	else {
		// add block to geode board
		getLevel().getBoard()->addDrawable(new osg::ShapeDrawable(new osg::Box(osg::Vec3(x, y, height / 2.0f), 1.0, 1.0, height)));
		getTeeter().PX::addBoxToCompound(getLevel().getBoard()->getId(), osg::Vec3(1.0, 1.0, height), "holz", osg::Matrix::translate(x, y, height / 2.0f), Level::Groups::BOARD, Level::Groups::BALL);
		transform = getLevel().getBoard()->getTransform();
	}
}

int Block::fixBlockOnBoard() {
	int jointId = getTeeter().PX::getNextJointId();
	getTeeter().PX::createFixedJoint(jointId, getId(), osg::Matrix::identity(), getLevel().getBoard()->getId(), transform->getMatrix());
	return jointId;
}