#pragma once

#include "Block.h"
#include <QObject>
#include <QTimer>

class Ball;

class BlockButton : public QObject, public Block{
Q_OBJECT

public:
	enum class Type {
		TRIGGER,
		PUSHBUTTON,
		SWITCH
	};

	/// <summary>
	/// Determines whether the specified hue describes a button block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 270</returns>
	static bool isHue(int hue) { return hue >= 260 && hue <= 280; }

	static BlockButton* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockButton(level, x, y, s, b, group); }

	void contactedBy(Ball& ball);

	void reset();

protected:
	BlockButton(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockButton() { }

private:
	Type type;

	void makeTrigger(int saturation);
	void makePushButton(int saturation);
	void makeSwitch(int saturation);

	bool activated;
	void setActivated(bool a);
	void triggerGroup();

	float coolDownDuration;
	bool coolsDown;

	void buildButton();

	ref_ptr<Geode> buttonGeode;
	ref_ptr<ShaderProgram> buttonShader;

	void animateColor(Vec4f from, Vec4f to, float duration);
	void animatePose(Vec3f from, Vec3f to, float duration);

	ref_ptr<Uniform> buttonPose;
	ref_ptr<Uniform> buttonPoseAnimationTarget;
	ref_ptr<Uniform> buttonPoseAnimationStart;
	ref_ptr<Uniform> buttonPoseAnimationEnd;
	Vec4f color;
	ref_ptr<Uniform> buttonColor;
	ref_ptr<Uniform> buttonColorAnimationTarget;
	ref_ptr<Uniform> buttonColorAnimationStart;
	ref_ptr<Uniform> buttonColorAnimationEnd;


private slots:
	void cooledDown();
};