#pragma once

#include "Block.h"
#include <unordered_set>

class Ball;

class BlockCheckpoint : public Block{
public:
	enum class Type {
		START,
		CHECKPOINT,
		END
	};

	/// <summary>
	/// Determines whether the specified hue describes a checkpoint block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 135</returns>
	static bool isHue(int hue) { return hue >= 125 && hue <= 145; }

	static BlockCheckpoint* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockCheckpoint(level, x, y, s, b, group); }

	void contactedBy(Ball& ball);

	void reset();

protected:
	BlockCheckpoint(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockCheckpoint();

private:
	Type type;

	void makeStart();
	void makeCheckpoint(bool mandatory, int pointOffset);
	void makeEnd(int pointOffset);
	void checkEnd();
	void awardPoints();

	void createFlag();
	ref_ptr<Geode> flagGeode;
	ref_ptr<ShaderProgram> flagShader;
	ref_ptr<Geode> flagstaffGeode;

	void animatePose(Vec3f from, Vec3f to, float duration);
	ref_ptr<Uniform> flagPose;
	ref_ptr<Uniform> flagPoseAnimationTarget;
	ref_ptr<Uniform> flagPoseAnimationStart;
	ref_ptr<Uniform> flagPoseAnimationEnd;

	bool mandatory;
	bool visited;
	int points;

	static std::unordered_set<BlockCheckpoint*> checkpoints;
};