#pragma once

#include "Block.h"
#include <QTimer>
#include <QObject>

#include <functional>

class Ball;

class BlockElevator : public QObject, public Block {
Q_OBJECT

public:

	/// <summary>
	/// Determines whether the specified hue describes a elevator block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 180</returns>
	static bool isHue(int hue) { return hue >= 170 && hue <= 190; }

	static BlockElevator* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockElevator(level, x, y, s, b, group); }

	void triggeredBy(Block& block);

	void reset();

	void frameEvent();

protected:
	BlockElevator(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockElevator();

private:
	int joint;
	int fixedJoint;
	bool fixed;

	QTimer timer;
	double movement;
	bool autoMovement;

	int direction;

	std::function<void()> frameFunction;

private slots:
	void move();

};