#include "BlockGhost.h"
#include "Teeter.h"
#include "Level.h"
#include "Board.h"

BlockGhost::BlockGhost(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group)
: Block(level, x, y, s, b, group, true, "Ghost")
{
	fixBlockOnBoard();

	int offset = s;
	if (s >= 79) { // ">= 80"
		initiallyVisible = true;
		offset -= 80;
	}
	else { // ">= 50", fallback
		initiallyVisible = false;
		offset -= 50;
	}
	offset = max(0, offset);
	if (offset < 17) {
		offset = ((int)(offset / 3.0 + 0.5)) * 3;
		duration = (offset + 3.0) / 6.0;
		timerFunction = std::bind(&BlockGhost::timerEvent, this);
		GameLoop::registerForTimedEvents(timerFunction, duration);
	}
	
	if (!getTeeter().shaders.count("ghostShader.frag")) {
		getTeeter().shaders["ghostShader.frag"] = new ShaderProgram::Shader("data/Teeter/ghostShader.frag", Shader::FRAGMENT);
		getTeeter().shaders["ghostShader.frag"]->addCodeFileInjection("data/classicnoise3D.glsl", "//HOTSPOT_PRE_MAIN");
		getTeeter().shaders["ghostShader.frag"]->addCodeFileInjection("data/wood.glsl", "//HOTSPOT_PRE_MAIN");
		getTeeter().shaders["ghostShader.frag"]->addCodeFileInjection("data/easeFunctions.glsl", "//HOTSPOT_PRE_MAIN");
		getTeeter().shaders["ghostShader.frag"]->addCodeFileInjection("data/phong_frag.glsl", "//HOTSPOT_PRE_MAIN");
	}

	if (!getLevel().getBoard()->getShaderGroup("Ghost")->shaderProgram) {
		ref_ptr<ShaderProgram> ghostShader = new ShaderProgram(getLevel().getBoard()->getShaderGroup("Ghost"));
		ghostShader->setVertexShader(getTeeter().shaders["boardShader.vert"]);
		ghostShader->setFragmentShader(getTeeter().shaders["ghostShader.frag"]);

		ref_ptr<StateSet> stateSet = getLevel().getBoard()->getShaderGroup("Ghost")->getOrCreateStateSet();
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		getLevel().getBoard()->getShaderGroup("Ghost")->shaderProgram = ghostShader.get();
	}
	
	ref_ptr<StateSet> stateSet = getTransform()->getOrCreateStateSet();
	opaque = new Uniform("a_Opaque", true);
	stateSet->addUniform(opaque);
	opaqueStart = new Uniform("a_OpaqueStart", 0.0f);
	stateSet->addUniform(opaqueStart);
	opaqueEnd = new Uniform("a_OpaqueEnd", 0.0f);
	stateSet->addUniform(opaqueEnd);

	reset();
}

void BlockGhost::triggeredBy(Block& block) {
	setVisible(!visible);	
}

void BlockGhost::setVisible(bool v) {
	visible = v;
	getTeeter().PX::setSimulateCollisions(getId(), visible);

	float time = getTeeter().getFrameStamp()->getSimulationTime();
	opaque->set(visible);
	opaqueStart->set(time);
	opaqueEnd->set(time + 0.5f);
}

void BlockGhost::timerEvent() {
	setVisible(!visible);
}

void BlockGhost::reset() {
	setVisible(initiallyVisible);
}

BlockGhost::~BlockGhost() {
	GameLoop::unregisterFromTimedEvents(timerFunction, duration);
}