#include "BlockItem.h"
#include "Teeter.h"
#include "Level.h"
#include "Board.h"
#include "Ball.h"

BlockItem::BlockItem(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group)
: Block(level, x, y, s, b, group, true, "Item")
, timer()
{ 
	fixBlockOnBoard();

	if (s >= 89) { // ">= 90"
		makeGold(offset(s, 90) / 3 + 1);
	}
	else if (s >= 79) { // ">= 80"
		makeGold(-offset(s, 80) / 3 - 1);
	}
	else if(s >= 69) { // ">= 70"
		makeLife(offset(s, 70) / 3 + 1);
	}
	else if(s >= 59) { // ">= 60"
		makeLife(-offset(s, 60) / 3 - 1);
	}
	else if(s >= 49) { // ">= 50"
		makeAntigravity(offset(s, 50));
	}
	else { // fallback
		makeGold(1);
	}
	getLevel().getBoard()->registerContactableBlock(this, getId());
}

int BlockItem::offset(int saturation, int baseSaturation) {
	int offset = max(0, saturation - baseSaturation);
	offset = ((int)(offset / 3.0 + 0.5)) * 3;
	return offset;
}


void BlockItem::makeGold(int num) {
	type = Type::GOLD;
	triggered = false;
	amount = num * 10;

	ref_ptr<Drawable> coin = dynamic_cast<osg::Geode*>(osgDB::readNodeFile("data/Teeter/coin.osgt"))->getDrawable(0);
	createItems(coin.get(), Vec4(1.0, 0.85, 0, 1), num);
}

void BlockItem::triggerGold() {
	if (triggered)
		return;
	triggered = true;
	getLevel().changePoints(amount);
	singleShot(10000, &BlockItem::reset);
}

void BlockItem::resetGold() {
	triggered = false;
}


void BlockItem::makeLife(int num) {
	type = Type::LIFE;
	triggered = false;
	amount = num;
	ref_ptr<Drawable> heart = dynamic_cast<osg::Geode*>(osgDB::readNodeFile("data/Teeter/heart.osgt"))->getDrawable(0);
	createItems(heart.get(), Vec4(0.9, 0, 0, 1), num);
}

void BlockItem::triggerLife() {
	if (triggered)
		return;
	triggered = true;
	getLevel().changeLives(amount, false);
	singleShot(10000, &BlockItem::reset);
}

void BlockItem::resetLife() {
	triggered = false;
}

void BlockItem::createItems(Drawable* item, Vec4 color, int num) {
	float groundHeight = getHeight() / 2 - (blockEndHeight - blockStartHeight) / 2;
	ref_ptr<MatrixTransform> mt = new MatrixTransform(osg::Matrix::scale(0.2, 0.2, 0.2) * osg::Matrix::translate(0, 0, groundHeight + 0.3));

	ref_ptr<Geode> geode = new Geode();
	geode->addDrawable(item);
	if (abs(num) > 0)
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_ItemDisplace", Vec3f(1, 0, 0.5)));
	mt->addChild(geode);

	if (abs(num) > 1) {
		geode = new Geode();
		geode->addDrawable(item);
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_ItemDisplace", Vec3f(0.7, 0.6, -0.4)));
		mt->addChild(geode);
	}

	if (abs(num) > 2) {
		geode = new Geode();
		geode->addDrawable(item);
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_ItemDisplace", Vec3f(1.4, -0.6, -0.2)));
		mt->addChild(geode);
	}
	if (abs(num) > 0) {
		geode = new Geode();
		geode->addDrawable(new ShapeDrawable(new osg::Box(Vec3(), 1, 0.2, 0.3)));
		if (num > 0) {
			geode->addDrawable(new ShapeDrawable(new osg::Box(Vec3(), 0.3, 0.2, 1)));
		}
		geode->getOrCreateStateSet()->addUniform(new Uniform("a_ItemDisplace", Vec3f(-1, 0, 0)));
		mt->addChild(geode);
	}

	if (!getTeeter().shaders.count("itemShader.vert")) {
		getTeeter().shaders["itemShader.vert"] = new ShaderProgram::Shader("data/Teeter/itemShader.vert", Shader::VERTEX);
		getTeeter().shaders["itemShader.vert"]->addCodeFileInjection("data/phong_vert.glsl", "//HOTSPOT_PRE_MAIN");
		getTeeter().shaders["itemShader.vert"]->addCodeFileInjection("data/rotation.glsl", "//HOTSPOT_PRE_MAIN");

		getTeeter().shaders["itemShader.frag"] = new ShaderProgram::Shader("data/Teeter/itemShader.frag", Shader::FRAGMENT);
		getTeeter().shaders["itemShader.frag"]->addCodeFileInjection("data/phong_frag.glsl", "//HOTSPOT_PRE_MAIN");
	}

	itemShader = new ShaderProgram(mt);
	itemShader->setVertexShader(getTeeter().shaders["itemShader.vert"]);
	itemShader->setFragmentShader(getTeeter().shaders["itemShader.frag"]);

	mt->getOrCreateStateSet()->addUniform(new Uniform("a_Color", color));
	mt->setCullCallback(new BlockItemCallback(this));

	getTransform()->addChild(mt.get());
}


void BlockItem::makeAntigravity(int offset) {
	type = Type::ANTIGRAVITY;
	triggered = false;
	duration = offset + 1;

	ref_ptr<Drawable> heart = dynamic_cast<osg::Geode*>(osgDB::readNodeFile("data/Teeter/weight.osgt"))->getDrawable(0);
	createItems(heart.get(), Vec4(0.7, 0.7, 0.7, 1), 0);
}

void BlockItem::triggerAntigravity() {
	if (triggered)
		return;
	triggered = true;
	getTeeter().PX::setForceResponsiveness(getLevel().getBall()->getId(), getTeeter().PX::ForceResponsiveness::NO_GRAVITY);
	singleShot(duration*1000, &BlockItem::antigravityCooldown);
}

void BlockItem::antigravityCooldown() {
	getTeeter().PX::setForceResponsiveness(getLevel().getBall()->getId(), getTeeter().PX::ForceResponsiveness::ALL);
	singleShot(10000, &BlockItem::reset);
}

void BlockItem::resetAntigravity() {
	getTeeter().PX::setForceResponsiveness(getLevel().getBall()->getId(), getTeeter().PX::ForceResponsiveness::ALL);
	triggered = false;
}


void BlockItem::contactedBy(Ball& ball) {
	switch (type)
	{
	case Type::GOLD:
		triggerGold();
		break;
	case Type::LIFE:
		triggerLife();
		break;
	case Type::ANTIGRAVITY:
		triggerAntigravity();
		break;
	default:
		break;
	}
}

void BlockItem::singleShot(int duration, void(BlockItem::*callback)()) {
	timer.stop();
	timer.setSingleShot(true);
	timer.setInterval(duration);
	timer.disconnect();
	connect(&timer, &QTimer::timeout, this, callback);
	timer.start();
}

void BlockItem::reset() {
	timer.stop();
	switch (type)
	{
	case Type::GOLD:
		resetGold();
		break;
	case Type::LIFE:
		resetLife();
		break;
	case Type::ANTIGRAVITY:
		resetAntigravity();
		break;
	default:
		break;
	}
}

void BlockItem::BlockItemCallback::operator()(Node* node, NodeVisitor* nv) {
	if (!item->triggered) {
		NodeCallback::operator()(node, nv);
	}
}