#pragma once

#include "Block.h"
#include <QObject>
#include <QTimer>
#include <osg/NodeCallback>
#include <functional>

class Ball;

class BlockItem : public QObject, public Block{
Q_OBJECT

public:
	enum class Type {
		GOLD,
		LIFE,
		ANTIGRAVITY
	};

	/// <summary>
	/// Determines whether the specified hue describes an item block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 45</returns>
	static bool isHue(int hue) { return hue >= 35 && hue <= 55; }

	static BlockItem* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockItem(level, x, y, s, b, group); }

	void contactedBy(Ball& ball);

	void frameEvent();

	void reset();

protected:
	BlockItem(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockItem() {}

private:
	Type type;

	void makeGold(int num);
	void triggerGold();
	void resetGold();

	void makeLife(int num);
	void triggerLife();
	void resetLife();

	void makeAntigravity(int offset);
	void triggerAntigravity();
	void antigravityCooldown();
	void resetAntigravity();

	void createItems(Drawable* item, Vec4 color, int num);
	
	int offset(int saturation, int baseSaturation);

	QTimer timer;
	void singleShot(int duration, void(BlockItem::*callback)());

	union{
		// Amount for Gold and Life
		double amount;
		// Duration for Antigravity
		double duration;
	};

	bool triggered;

	osg::ref_ptr<ShaderProgram> itemShader;

	class BlockItemCallback : public osg::NodeCallback {
	public:
		BlockItemCallback(BlockItem* item) : item(item) {}
		void operator()(Node* node, NodeVisitor* nv);
	private:
		BlockItem* item;
	};
};