#pragma once

#include "Block.h"
#include <QObject>
#include <QTimer>
#include <osg/NodeCallback>
#include <osg/BlendFunc>
#include <functional>

#include <osgParticle/ParticleSystem>
#include <osgParticle/ParticleSystemUpdater>
#include <osgParticle/ModularEmitter>
#include <osgParticle/MultiSegmentPlacer>
#include <osgParticle/ModularProgram>
#include <osgParticle/FluidFrictionOperator>

class Ball;

class BlockPortal : public Block{
public:
	enum class Orientation {
		RIGHT,
		LEFT,
		BACK,
		FRONT,
		TOP
	};

	/// <summary>
	/// Determines whether the specified hue describes a portal block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 315</returns>
	static bool isHue(int hue) { return hue >= 305 && hue <= 325; }

	static BlockPortal* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockPortal(level, x, y, s, b, group); }

	void triggeredBy(Ball& ball);
	void untriggeredBy(Ball& ball);

	void frameEvent();

	void reset();

protected:
	BlockPortal(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockPortal();

private:
	Orientation orientation;
		
	int offset(int saturation, int baseSaturation);
	void createParticles();

	enum class Progress {
		WAIT,
		ATTRACT,
		SEND,
		RECEIVE,
		REPEL
	};

	Progress progress;
	void startAttract();
	void attractBall();
	void startSend();
	BlockPortal* randomPartner();
	BlockPortal* partner;
	void sendBall();
	void startReceive();
	void sendComplete();
	void startRepel();

	void makePortal(int offset);
	void makeTrigger();
	float attraction;

	Vec3 entranceCenter;
	Vec3 entranceDirection;
	std::function<void()> frameFunction;

	// For the portal
	osg::ref_ptr<Cylinder> cylinder;

	osg::ref_ptr<ShaderProgram> shader;
	class BallPortalShaderCallback : public osg::Uniform::Callback
	{
	public:
		BallPortalShaderCallback(BlockPortal* portal) : portal(portal) {}
		void operator() (osg::Uniform* uniform, osg::NodeVisitor* nv);
	private:
		ref_ptr<BlockPortal> portal;
	};
};