#pragma once

#include "Block.h"
#include <QObject>
#include <QTimer>
#include <osg/NodeCallback>
#include <functional>

#include <osg/BlendFunc>
#include <osgParticle/ParticleSystem>
#include <osgParticle/ParticleSystemUpdater>
#include <osgParticle/ModularEmitter>
#include <osgParticle/MultiSegmentPlacer>
#include <osgParticle/ModularProgram>
#include <osgParticle/FluidFrictionOperator>

class Ball;

class BlockTrap : public QObject, public Block{
Q_OBJECT

public:
	enum class Type {
		MINE,
		SPRING,
		TRAPDOOR,
		BLACK_HOLE
	};

	/// <summary>
	/// Determines whether the specified hue describes a trap block.
	/// </summary>
	/// <param name="hue">The hue.</param>
	/// <returns>true if the hue is about 0 (=360)</returns>
	static bool isHue(int hue) { return hue >= -10 && hue <= 10 || hue >= 350 && hue <= 370; }

	static BlockTrap* factory(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group) { return new BlockTrap(level, x, y, s, b, group); }

	void contactedBy(Ball& ball);
	void triggeredBy(Ball& ball);
	void untriggeredBy(Ball& ball);

	void frameEvent();

	void reset();

protected:
	BlockTrap(Level& level, float x, float y, int s, int b, std::vector<ref_ptr<Block>>& group);
	~BlockTrap();

private:
	Type type;

	void makeMine(int offset);
	void triggerMine();
	void hideMine();
	void mineExplode();
	void resetMine();

	void makeSpring(int offset);
	void triggerSpring();
	void resetSpring();

	void makeTrapdoor(int offset);
	void triggerTrapdoor();
	void trapdoorOpen();
	void resetTrapdoor();

	void makeBlackHole(int offset);
	void createBlackHoleParticles();
	void triggerBlackHole();
	void untriggerBlackHole();
	void blackHoleAttract();
	void resetBlackHole();

	int offset(int saturation, int baseSaturation);

	QTimer timer;
	void singleShot(int duration, void(BlockTrap::*callback)());

	union{
		// Delay for Mines and Trapdoors
		double delay;
		// Strength for Springs and Black Holes
		double strength;
	};

	union{
		// bool for Trapdoor
		bool open;
		// bool for Mine
		bool exploded;
		// bool for Black Hole
		bool entered;
	};

	// For mines, springs and trapdoors
	bool triggered;

	// For black holes
	std::function<void()> frameFunction;

	// For springs and black holes
	osg::ref_ptr<Cylinder> cylinder;

	// For mine
	osg::ref_ptr<Sphere> sphere;

	osg::ref_ptr<ShaderProgram> shader;

	class BlockTrapdoorCallback : public osg::NodeCallback {
	public:
		BlockTrapdoorCallback(BlockTrap* trap) : trap(trap) {}
		void operator()(Node* node, NodeVisitor* nv);
	private:
		BlockTrap* trap;
	};

	osg::ref_ptr<BlockTrapdoorCallback> callback;
};