#pragma once

#include <vector>

#include "GameObject.h"
#include "../physx/PhysxScene.h"
#include "Block.h"

class Ball;
class QImage;
class ShaderProgram;

class Board : public GameObject{

public:
	Board(Level& level);
	void build(QImage &image);
	void addDrawable(osg::Drawable* drawable) { board->addDrawable(drawable); }

	ShaderProgram::ShaderGroup* getShaderGroup(std::string type);

	MatrixTransform* getTransform() { return boardTransform.get(); }

	void reset(bool hardreset);
	void onContact(const physx::PxContactPairHeader& pairHeader, const physx::PxContactPair* pairs, physx::PxU32 nbPairs);
	void onTrigger(const physx::PxTriggerPair& pair, physx::PxRigidActor* const actors[2]);

	void registerContactableBlock(Block* block, int actorId);
	void registerTriggerableBlock(Block* block, int actorId);

	int getSynchronizerId() { return synchronizerId; }

protected:
	~Board() { }

private:
	ref_ptr<Group> group;
	ref_ptr<MatrixTransform> boardTransform;
	ref_ptr<Geode> board;
	std::vector<ref_ptr<Block>> blocks;
	std::unordered_map<int, std::vector<ref_ptr<Block>>> idGroups;

	std::unordered_map<std::string, ref_ptr<ShaderProgram::ShaderGroup>> shaderGroups;

	std::unordered_map<int, ref_ptr<Block>> contactableBlocks;
	std::unordered_map<int, ref_ptr<Block>> triggerableBlocks;
	
	ref_ptr<ShaderProgram> boardShader;

	ref_ptr<Geode> synchronizer;
	int synchronizerId;

	class SynchronizerCallback : public osg::NodeCallback {
	public:
		SynchronizerCallback(Teeter* teeter) : teeter(teeter) {}
		void operator()(Node* node, NodeVisitor* nv);
	private:
		Teeter* teeter;
	};
};