#include "Level.h"
#include "Teeter.h"
#include "Ground.h"
#include "Board.h"
#include "Ball.h"

Level::Level(int level, Group* parent, Teeter& teeter)
: level(level)
, parent(parent)
, teeter(teeter)
, lives(5)
, points(0)
{
	GameLoop::lock();
	nextId = 0;
	teeter.PX::createMaterial("holz", 0.2f, 0.05f, 0.f);

	std::stringstream sstm;
	sstm << "data/Teeter/level" << level << ".png";
	std::cout << "reading level " << sstm.str() << std::endl;
	group = new Group();
	shader = new ShaderProgram(group);
	if (!getTeeter().shaders.count("standardShader.vert"))
		getTeeter().shaders["standardShader.vert"] = new ShaderProgram::Shader("data/Teeter/standardShader.vert", Shader::VERTEX);
	shader->setVertexShader(getTeeter().shaders["standardShader.vert"]);
	if (!getTeeter().shaders.count("standardShader.frag"))
		getTeeter().shaders["standardShader.frag"] = new ShaderProgram::Shader("data/Teeter/standardShader.frag", Shader::FRAGMENT);
	shader->setFragmentShader(getTeeter().shaders["standardShader.frag"]);

	ground = new Ground(*this);
	board = new Board(*this);
	board->build(QImage(QString::fromStdString(sstm.str())));
	ball = new Ball(*this);

	livesHUD = new ImageHUD();
	livesHUD->setSourceFile("data/Teeter/lives-5.png");
	livesHUD->setRelativePosition(Vec2(0.9,0.97));
	livesHUD->setRelativeDimension(Vec2(0,0.03));
	livesHUD->setAlignment(osgText::TextBase::AlignmentType::RIGHT_TOP);
	livesHUD->attachToScene(group, &getTeeter());
	livesHUD->show();

	coinHUD = new ImageHUD();
	coinHUD->setSourceFile("data/Teeter/coins.png");
	coinHUD->setRelativePosition(Vec2(0.95, 0.97));
	coinHUD->setRelativeDimension(Vec2(0, 0.03));
	coinHUD->setAlignment(osgText::TextBase::AlignmentType::RIGHT_TOP);
	coinHUD->attachToScene(group, &getTeeter());
	coinHUD->show();

	pointsHUD = new ScoreHUD();
	pointsHUD->setRelativePosition(Vec2(0.955, 0.965));
	pointsHUD->setAlignment(osgText::TextBase::AlignmentType::LEFT_TOP);
	pointsHUD->attachToScene(group, &getTeeter());
	pointsHUD->show();

	parent->addChild(group);
	GameLoop::unlock();
}

void Level::won() {
	getTeeter().won();
}

void Level::lost() {
	getTeeter().lost();
}

void Level::setStartPoint(BlockCheckpoint* start) {
	startPoint = start;
	setLastCheckpoint(start);
}

void Level::changeLives(int l, bool resetLevel) {
	lives += l;
	lives = clampBetween(lives, 0, 5);
	std::stringstream sstm;
	sstm << "data/Teeter/lives-" << lives << ".png";
	livesHUD->setSourceFile(sstm.str());
	if (resetLevel || lives == 0)
		reset(lives == 0);
}

void Level::changePoints(int p) {
	p = max(-points, p);
	points += p;
	pointsHUD->setScore(points);
}

void Level::reset(bool hardreset) {
	if (hardreset) {
		lastCheckpoint = startPoint.get();
		lives = 5;
		std::stringstream sstm;
		sstm << "data/Teeter/lives-" << lives << ".png";
		livesHUD->setSourceFile(sstm.str());
		pointsHUD->setScore(0);
		points = 0;
	}
	ball->reset();
	board->reset(hardreset);

	getTeeter().setState(Teeter::LevelState::SYNCHRONIZING);

	if (hardreset)
		lost();
}

void Level::addChild(Node* node) {
	group->addChild(node);
}

void Level::removeChild(Node* node) {
	group->removeChild(node);
}

Level::~Level(){
	GameLoop::lock();
	parent->removeChild(group);
	teeter.PX::clearPhysxScene();
	GameLoop::unlock();
}