#pragma once

#include <osg/Referenced>
#include <osg/ref_ptr>
#include <osg/Vec3>
#include <osg/Group>

#include "BlockCheckpoint.h"
#include "../osg/ImageHUD.h"
#include "../osg/TextHUD.h"
#include "../osg/ScoreHUD.h"

class Board;
class Ground;
class Ball;
class Teeter;
class ShaderProgram;

using namespace osg;

class Level : public Referenced{
public:
	Level(int level, Group* parent, Teeter& teeter);

	enum Groups {
		GROUND = 1 << 0,
		BOARD = 1 << 1,
		BALL = 1 << 2
	};

	Teeter& getTeeter() { return teeter; }
	Board* getBoard() { return board.get(); }
	Ground* getGround() { return ground.get(); }
	Ball* getBall() { return ball.get(); }
	void addChild(Node* node);
	void removeChild(Node* node);

	int getNextId() { return nextId++; }
	int getLevel() { return level; }

protected:
	~Level();

private:
	Teeter& teeter;
	ref_ptr<Group> parent;
	ref_ptr<Group> group;

	ref_ptr<Board> board;
	ref_ptr<Ground> ground;
	ref_ptr<Ball> ball;

	ref_ptr<ShaderProgram> shader;

private:
	int points;
	int lives;
	ref_ptr<BlockCheckpoint> lastCheckpoint;
	ref_ptr<BlockCheckpoint> startPoint;

	ref_ptr<ImageHUD> livesHUD;
	ref_ptr<ImageHUD> coinHUD;
	ref_ptr<ScoreHUD> pointsHUD;

public:
	void won();
	void lost();
	void changePoints(int p);
	void setStartPoint(BlockCheckpoint* start);
	void setLastCheckpoint(BlockCheckpoint* checkpoint) { lastCheckpoint = checkpoint; }
	BlockCheckpoint* getLastCheckpoint() { return lastCheckpoint.get(); }

	void changeLives(int l, bool reset);
	void reset(bool hardreset = true);

private:
	int level;
	int nextId;
};