#pragma once

#include "../TeetairApplication.h"
#include "../osg/HandCamera.h"
#include "../osg/MenuHUD.h"
#include "../osg/TextHUD.h"

#include <osg/MatrixTransform>
#include <osgGA/TrackballManipulator>

#include <unordered_map>

class Level;

class Teeter : public TeetairApplication{
public:
	Teeter(OsgViewerWidget* osgViewerWidget);

	virtual void onFrame(const Controller&);
	virtual void onConnect(const Controller&);

	virtual void keyPressEvent(QKeyEvent* event);
	virtual void keyReleaseEvent(QKeyEvent* event);

	virtual void onContact(const physx::PxContactPairHeader& pairHeader, const physx::PxContactPair* pairs, physx::PxU32 nbPairs);
	virtual void onTrigger(PxTriggerPair* pairs, PxU32 count);

	virtual void frameEvent();
	virtual float simulationStep();
	virtual int fps();

	void setTransform(int id, osg::MatrixTransform* transform);
	void removeTransform(int id);
	
	std::unordered_map<std::string, ref_ptr<ShaderProgram::Shader>> shaders;

public:
	void start();
	void won();
	void lost();

	enum class LevelState {
		START,
		CAMERA,
		SYNCHRONIZING,
		RUNNING,
		PAUSE,
		WON,
		LOST
	};
	LevelState getState() { return state; }
	void setState(LevelState s) { state = s; }

private:
	ref_ptr<MenuHUD> pauseMenu;
	ref_ptr<MenuHUD> endMenu;
	ref_ptr<TextHUD> notification;
	ref_ptr<Level> level;
	void loadLevel(int levelNumber);

	enum RequestType {
		NONE,
		RESTART,
		NEXT,
		PREVIOUS
	} requested = RequestType::NONE;

	LevelState state;

private:
	void createScene();

	boolean keyUpPressed = false, keyDownPressed = false, keyLeftPressed = false, keyRightPressed = false;
	ref_ptr<HandCamera> handCamera;
	void rotateBoard(Leap::Frame& frame);
	void synchronize(Leap::Frame& frame);
	void pause(const Controller& controller);
	void end(const Controller& controller);
	bool openPauseMenu(Leap::Frame& frame);
};