#include "FmodScene.h"


bool FmodScene::addSound(const string name, const string file) {
	Sound *sound;
	result = system()->createSound(file.c_str(), FMOD_DEFAULT, 0, &sound);
	if (!checkResult(result)) return false;
	sounds[name] = sound;
	return true;
}

bool FmodScene::playSound(const string name, float volume, playmode mode, int *channelId) {
	if (!sounds.count(name)){
		printf("I do not know the sound %s !\n", name.c_str());
		return false;
	}
	Channel *channel;
	result = system()->playSound(sounds[name], 0, true, &channel);
	if (!checkResult(result)) return false;
	int id = -1;
	
	switch (mode) {
	case loop:
		result = channel->setMode(FMOD_LOOP_NORMAL);
		if (!checkResult(result)) return false;
		break;
	case shot:
	default:
		result = channel->setMode(FMOD_LOOP_OFF);
		if (!checkResult(result)) return false;
		break;
	}

	result = channel->setVolume(volume);
	if (!checkResult(result)) return false;
	result = channel->setPaused(false);
	if (!checkResult(result)) return false;
	
	if (channelId != NULL && mode != shot) {
		channel->getIndex(channelId);
		channels[*channelId] = channel;
	}

	return true;
}

bool FmodScene::pauseSound(int channelId) {
	if (!channels.count(channelId)) {
		cout << "I do not have a channel #" << channelId << endl;
		return false;
	}
	result = channels[channelId]->setPaused(true);
	if (!checkResult(result)) return false;
	return true;
}

bool FmodScene::unpauseSound(int channelId) {
	if (!channels.count(channelId)) {
		cout << "I do not have a channel #" << channelId << endl;
		return false;
	}
	result = channels[channelId]->setPaused(false);
	if (!checkResult(result)) return false;
	return true;
}

bool FmodScene::stopSound(int channelId) {
	if (!channels.count(channelId)) {
		cout << "I do not have a channel #" << channelId << endl;
		return false;
	}
	result = channels[channelId]->stop();
	if (!checkResult(result)) return false;
	channels.erase(channelId);
	return true;
}

bool FmodScene::checkResult(FMOD_RESULT result) {
	if (result != FMOD_OK) {
		cout << "FMOD error " << result << " - " << FMOD_ErrorString(result) << endl;
		return false;
	}
	return true;
}

System* FmodScene::system() {
	return FmodManager::system();
}

FmodScene::~FmodScene() {
	for (unordered_map<string, Sound*>::iterator i = sounds.begin(); i != sounds.end(); ++i) {
		i->second->release();
	}
}