#include "AnimatedTextHUD.h"

AnimatedTextHUD::AnimatedTextHUD()
: TextHUD()
, _timer(this)
{
	_relativePosition = osg::Vec2(0.5, 0.1);
	_alignment = osgText::TextBase::AlignmentType::CENTER_BOTTOM;
	connect(&_timer, SIGNAL(timeout()), this, SLOT(hide()));
	_timer.setSingleShot(true);
	setTimes(3, 7, 3);
}

void AnimatedTextHUD::addShader() {
	ShaderProgram* textShader = new ShaderProgram(_geode);

	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("GlyphTexture", 0));
	// Pass timing values to the shader
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("a_timeIn", _timeIn));
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("a_timeUp", _timeUp));
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("a_timeOut", _timeOut));

	textShader->setVertexShader("data/animatedTextShader.vert");
	textShader->setFragmentShader("data/animatedTextShader.frag");
}

void AnimatedTextHUD::setTimes(float timeIn, float timeUp, float timeOut) {
	_timeIn = timeIn;
	_timeUp = timeUp;
	_timeOut = timeOut;
	float totalTime = timeIn + timeUp + timeOut;
	_timer.setInterval(totalTime * 1000);
	if (_geode != nullptr)
		addShader();
}

void AnimatedTextHUD::show() {
	TextHUD::show();

	// Pass new start time to the shader
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("a_startTime", (float)(_view->getFrameStamp()->getSimulationTime())));
	
	_timer.start();
}

// Needed for the timer
void AnimatedTextHUD::hide() {
	TextHUD::hide();
}