#include "ContainerHUD.h"

ContainerHUD::ContainerHUD()
{
	_group = new osg::Group;
	// Needs to have the same attributes (rendered last etc.)

	// For this state set, turn blending on (so alpha texture looks right)
	_group->getOrCreateStateSet()->setMode(GL_BLEND, osg::StateAttribute::ON);

	// Disable depth testing so geometry is draw regardless of depth values
	// of geometry already draw.
	_group->getOrCreateStateSet()->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
	_group->getOrCreateStateSet()->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

	// Need to make sure this geometry is draw last. RenderBins are handled
	// in numerical order so set bin number to 11
	_group->getOrCreateStateSet()->setRenderBinDetails(11, "RenderBin");

	HUDModelViewMatrix->addChild(_group);
}

void ContainerHUD::clear() {
	for (int i = 0; i < _huds.size(); i++) {
		_huds.at(i)->clear();
	}
}

void ContainerHUD::addChild(AbstractHUD* hud) {
	osg::ref_ptr<osg::Geode> childGeode = new osg::Geode;
	hud->setGeode(childGeode);
	// Save the reference to prevent destruction
	_huds.push_back(hud);

	hud->setResolution(_width, _height);

	if (visible)
		hud->show();
	else
		hud->hide();
}

void ContainerHUD::showHUD(AbstractHUD* hud) {
	// Add the corresponding node
	_group->addChild(hud->getGeode());
}

void ContainerHUD::hideHUD(AbstractHUD* hud) {
	// Remove the corresponding node
	_group->removeChild(hud->getGeode());
}


void ContainerHUD::draw() {

}

void ContainerHUD::deleteHUDs() {
	_huds.clear();
}

void ContainerHUD::show() {
	AbstractHUD::show();
	for (int i = 0; i < _huds.size(); i++) {
		_huds.at(i)->show();
	}
}

void ContainerHUD::hide() {
	AbstractHUD::hide();
	for (int i = 0; i < _huds.size(); i++) {
		_huds.at(i)->hide();
	}
}

void ContainerHUD::setResolution(int width, int height) {
	AbstractHUD::setResolution(width, height);
	for (int i = 0; i < _huds.size(); i++) {
		_huds.at(i)->setResolution(width, height);
	}
}