#include "HandCamera.h"

HandCamera::HandCamera()
: osgGA::TrackballManipulator()
{
	_height = 0;
	_distance = 10;
	_angle = 0;
	_center = osg::Vec3();
}

void HandCamera::setHeight(float height) {
	_height = height;
	updatePosition();
}

void HandCamera::changeHeight(float diff) {
	setHeight(_height + diff);
}

float HandCamera::getHeight() {
	return _height;
}

void HandCamera::setAngle(float angle) {
	_angle = angle;
	updatePosition();
}

void HandCamera::changeAngle(float diff) {
	setAngle(_angle + diff);
}

float HandCamera::getAngle() {
	return _angle;
}

void HandCamera::setCenter(osg::Vec3 center) {
	_center = center;
	updatePosition();
}

osg::Vec3 HandCamera::getCenter() {
	return _center;
}

void HandCamera::setDistance(float distance) {
	_distance = distance;
	updatePosition();
}

float HandCamera::getDistance() {
	return _distance;
}

void HandCamera::updatePosition() {
	setHomePosition(osg::Vec3(_distance * sin(_angle), _distance * cos(_angle), _height), _center, osg::Vec3(0.0, 0.0, 1.0));
	home(0);
}

bool HandCamera::handleLeapFrame(const Leap::Frame frame) {
	if (frame.hands().count() > 1) {
		// control the viewing angle
		float angleThreshold = 0.01f;
		float angleScale = 1.f / 60.f; // scale for the angle change calculation
		float angleChange = (frame.hands()[0].direction().yaw() + frame.hands()[1].direction().yaw()) * angleScale; // average hand rotation

		if (abs(angleChange) > angleThreshold)
			changeAngle(angleChange);

		// control the viewing height
		float cameraThreshold = 0.2f;
		float standardHeight = 200; // height of hands in mm where the camera height isn't changed
		float cameraHeightScale = 1.f / 300.f; // scale for the height change calculation
		float cameraHeightChange = (frame.hands()[0].palmPosition().y + frame.hands()[1].palmPosition().y - standardHeight * 2) * cameraHeightScale; // average hand position

		if (abs(cameraHeightChange) > cameraThreshold)
			changeHeight(cameraHeightChange);
		return true;
	}
	return false;
}