#include "ImageHUD.h"

ImageHUD::ImageHUD(){

}
void ImageHUD::addShader() {
	ShaderProgram* imageShader = new ShaderProgram(_geode);
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("imageTexture", 0));
	imageShader->setVertexShader("data/imageShader.vert");
	imageShader->setFragmentShader("data/imageShader.frag");
}

void ImageHUD::draw() {
	// Set up a screen quad for the image
	osg::ref_ptr<osg::Geometry> screenQuad;
	osg::ref_ptr<osg::Image> image = osgDB::readImageFile(_fileName);
	osg::Texture2D *texture = new osg::Texture2D;
	texture->setImage(image);

	

	int absoluteX = _relativePosition.x() * getWidth();
	int absoluteY = _relativePosition.y() * getHeight();

	int absoluteDX = _relativeDimension.x() * getWidth();
	int absoluteDY = _relativeDimension.y() * getHeight();

	if (absoluteDX == 0) {
		if (absoluteDY == 0) {
			absoluteDX = image->s();
			absoluteDY = image->t();
		}
		else {
			absoluteDX = absoluteDY * image->s() / image->t();
		}
	}
	else {
		if (absoluteDY == 0) {
			absoluteDY = absoluteDX * image->t() / image->s();
		}
	}

	osg::Vec2 position = osg::Vec2(absoluteX, absoluteY) - offset(absoluteDX, absoluteDY);

	screenQuad = osg::createTexturedQuadGeometry(osg::Vec3(position, 0),
		osg::Vec3(absoluteDX, 0.0, 0.0),
		osg::Vec3(0.0, absoluteDY, 0.0));
	screenQuad->getOrCreateStateSet()->setTextureAttributeAndModes(0, texture, osg::StateAttribute::ON);
	_geode->addDrawable(screenQuad);
}

osg::Vec2 ImageHUD::offset(int dX, int dY) {
	float offsetX = dX * (floor(_alignment / 3) / 2.f);
	float offsetY = dY * ((2.f -_alignment % 3) / 2.f);
	return osg::Vec2(offsetX, offsetY);
}

void ImageHUD::setRelativePosition(osg::Vec2 relativePosition) {
	_relativePosition = relativePosition;
	redraw();
}

void ImageHUD::setSourceFile(std::string fileName) {
	_fileName = fileName;
	redraw();
}

void ImageHUD::setRelativeDimension(osg::Vec2 relativeDimension) {
	_relativeDimension = relativeDimension;
	redraw();
}

void ImageHUD::setAlignment(enum osgText::TextBase::AlignmentType alignment) {
	_alignment = alignment;
	redraw();
}

void ImageHUD::attachToScene(osg::Group* node, osgViewer::View* view) {
	AbstractHUD::attachToScene(node, view);
	addShader();
}

void ImageHUD::attachToContainer(ContainerHUD* container) {
	AbstractHUD::attachToContainer(container);
	addShader();
}