#pragma once

#include <osgGA/GUIEventHandler>
#include "KeyListener.h"

#include "../GameLoop.h"
#include "../leap/LeapManager.h"

#include "ContainerHUD.h"

class MenuItemFolder;

class MenuHUD : public ContainerHUD {
public:
	MenuHUD();
	void show();
	void hide();
	void draw();
	void setCurrentMenu(MenuItemFolder*);
	void setMainMenu(MenuItemFolder*);
	void onFrame(Controller);
	void keyReleaseEvent(QKeyEvent*);
protected:
	MenuItemFolder* _mainMenu;
	MenuItemFolder* _currentMenu = nullptr;
};

class AbstractMenuItem {
public:
	AbstractMenuItem(char*, char*);
	virtual void select(MenuHUD*) = 0;
	char* getText();
	char* getImage();
protected:
	char* _text;
	char* _image;
};

class MenuItemAction : public AbstractMenuItem {
public:
	MenuItemAction(std::function<void()>, char*, char* = NULL);
	void select(MenuHUD*);
protected:
	std::function<void()> _f;
};

class MenuItemFolder : public AbstractMenuItem {
public:
	MenuItemFolder(char*, char* = NULL);
	void addItem(AbstractMenuItem*);
	void onFrame(const Controller& controller);
	void keyReleaseEvent(QKeyEvent*);
	void select(MenuHUD*);
	std::vector<AbstractMenuItem*> subItems;
	int counter = 0;
	int highlighted = 0;
protected:
	void selectNext();
	void selectPrevious();
	void callSelected();
	MenuHUD* _hud = NULL;
};