#include "OsgViewerWidget.h"


OsgViewerWidget::OsgViewerWidget()
	:keyListeners(NULL)
	,mouseListeners(NULL)
{
	setThreadingModel(osgViewer::CompositeViewer::SingleThreaded);
	
	setKeyEventSetsDone(0); // disable the default setting of viewer.done() by pressing Escape.
	new osgQt::GraphicsWindowQt(this);
	getGraphicsContext()->getState()->setUseModelViewAndProjectionUniforms(true);
	getGraphicsContext()->getState()->setUseVertexAttributeAliasing(true);
}

GraphicsContext* OsgViewerWidget::getGraphicsContext() const {
	return const_cast<osgQt::GraphicsWindowQt*>(getGraphicsWindow());
}

void OsgViewerWidget::paintEvent(QPaintEvent* event) {
	frame();
}

void OsgViewerWidget::registerKeyListener(KeyListener *listener){
	keyListeners.push_back(listener);
}
void OsgViewerWidget::unregisterKeyListener(KeyListener *listener){
	keyListeners.remove(listener);
}
void OsgViewerWidget::registerMouseListener(MouseListener *listener){
	mouseListeners.push_back(listener);
}
void OsgViewerWidget::unregisterMouseListener(MouseListener *listener){
	mouseListeners.remove(listener);
}

void OsgViewerWidget::keyPressEvent(QKeyEvent* event){
	GLWidget::keyPressEvent(event);
	event->setAccepted(false);
	for (std::list<KeyListener*>::iterator i = keyListeners.begin(); i != keyListeners.end() && !event->isAccepted(); ++i) {
		(*i)->keyPressEvent(event);
	}
}

void OsgViewerWidget::keyReleaseEvent(QKeyEvent* event) {
	GLWidget::keyReleaseEvent(event);
	event->setAccepted(false);
	switch (event->key())
	{
	case Qt::Key_F11:
		if (isFullScreen()) {
			showNormal();
		} else {
			showFullScreen();
		}
		event->accept();
		return;
	}
	for (std::list<KeyListener*>::iterator i = keyListeners.begin(); i != keyListeners.end() && !event->isAccepted(); ++i) {
		(*i)->keyReleaseEvent(event);
	}
}

void OsgViewerWidget::mousePressEvent(QMouseEvent* event){
	GLWidget::mousePressEvent(event);
	event->setAccepted(false);
	for (std::list<MouseListener*>::iterator i = mouseListeners.begin(); i != mouseListeners.end() && !event->isAccepted(); ++i) {
		(*i)->mousePressEvent(event);
	}
}
void OsgViewerWidget::mouseReleaseEvent(QMouseEvent* event){
	GLWidget::mouseReleaseEvent(event);
	event->setAccepted(false);
	for (std::list<MouseListener*>::iterator i = mouseListeners.begin(); i != mouseListeners.end() && !event->isAccepted(); ++i) {
		(*i)->mouseReleaseEvent(event);
	}
}
void OsgViewerWidget::mouseDoubleClickEvent(QMouseEvent* event){
	GLWidget::mouseDoubleClickEvent(event);
	event->setAccepted(false);
	for (std::list<MouseListener*>::iterator i = mouseListeners.begin(); i != mouseListeners.end() && !event->isAccepted(); ++i) {
		(*i)->mouseDoubleClickEvent(event);
	}
}
void OsgViewerWidget::mouseMoveEvent(QMouseEvent* event){
	GLWidget::mouseMoveEvent(event);
	event->setAccepted(false);
	for (std::list<MouseListener*>::iterator i = mouseListeners.begin(); i != mouseListeners.end() && !event->isAccepted(); ++i) {
		(*i)->mouseMoveEvent(event);
	}
}
void OsgViewerWidget::wheelEvent(QWheelEvent* event){
	GLWidget::wheelEvent(event);
	event->setAccepted(false);
	for (std::list<MouseListener*>::iterator i = mouseListeners.begin(); i != mouseListeners.end() && !event->isAccepted();++i) {
		(*i)->wheelEvent(event);
	}
}