#include "TextHUD.h"

void TextHUD::attachToScene(osg::Group* node, osgViewer::View* view) {
	AbstractHUD::attachToScene(node, view);
	addShader();
}

void TextHUD::attachToContainer(ContainerHUD* container) {
	AbstractHUD::attachToContainer(container);
	addShader();
}

void TextHUD::addShader() {
	ShaderProgram* textShader = new ShaderProgram(_geode);
	_geode->getOrCreateStateSet()->addUniform(new osg::Uniform("GlyphTexture", 0));
	textShader->setVertexShader("data/textShader.vert");
	textShader->setFragmentShader("data/textShader.frag");
}

void TextHUD::draw() {
	osgText::Text* textOne = new osgText::Text();
	_geode->addDrawable(textOne);

	// Set up the parameters for the text we'll add to the HUD
	textOne->setCharacterSize(_size);
	textOne->setFont(osgText::readFontFile("data/Bevan.ttf"));
	textOne->setText(_text);
	textOne->setBackdropColor(osg::Vec4(1, 1, 1, 1));
	textOne->setBackdropOffset(0.05);
	textOne->setBackdropImplementation(osgText::Text::BackdropImplementation::STENCIL_BUFFER);
	textOne->setBackdropType(osgText::Text::BackdropType::OUTLINE);
	textOne->setAxisAlignment(osgText::Text::SCREEN);
	int absoluteX = _relativePosition.x() * getWidth();
	int absoluteY = _relativePosition.y() * getHeight();
	textOne->setPosition(osg::Vec3(absoluteX, absoluteY, 0));
	textOne->setColor(_color);
	textOne->setAlignment(_alignment);
}

void TextHUD::setSize(float size) {
	_size = size;
	redraw();
}

void TextHUD::setText(std::string text) {
	_text = text;
	redraw();
}

void TextHUD::setRelativePosition(osg::Vec2 relativePosition) {
	_relativePosition = relativePosition;
	redraw();
}

void TextHUD::setAlignment(enum osgText::TextBase::AlignmentType alignment) {
	_alignment = alignment;
	redraw();
}

void TextHUD::setColor(osg::Vec4 color) {
	_color = color;
	redraw();
}