#include "PhysxManager.h"

PhysxManager* PhysxManager::instance() {
	static osg::ref_ptr<PhysxManager> manager = new PhysxManager;
	return manager.get();
}

PxPhysics* PhysxManager::getPhysics() {
	return instance()->physics;
}

PxCooking* PhysxManager::getCooking() {
	return instance()->cooking;
}

PhysxManager::PhysxManager()
: foundation(NULL), profileZoneManager(NULL), physics(NULL)
{
	static PxDefaultErrorCallback defaultErrorCallback;
	static PxDefaultAllocator defaultAllocatorCallback;

	foundation = PxCreateFoundation(PX_PHYSICS_VERSION, defaultAllocatorCallback, defaultErrorCallback);
	if (!foundation) {
		std::cout << "Unable to initialize PhysX foundation" << std::endl;
		return;
	}

	bool readMemoryAllocations = true;
	profileZoneManager = &PxProfileZoneManager::createProfileZoneManager(foundation);
	if (!profileZoneManager) {
		std::cout << "PxProfileZoneManager::createProfileZoneManager failed!" << std::endl;
		return;
	}
	
	physics = PxCreatePhysics(PX_PHYSICS_VERSION, *foundation, PxTolerancesScale(), true, profileZoneManager);
	if (!physics) {
		std::cout << "Unable to initialize PhysX SDK" << std::endl;
		return;
	}

	cooking = PxCreateCooking(PX_PHYSICS_VERSION, *foundation, PxCookingParams(physics->getTolerancesScale()));
	if (!cooking) {
		std::cout << "Unable to initialize PhysX cooking" << std::endl;
		return;
	}
	
	if (!PxInitExtensions(*physics)) {
		std::cout << "Unable to initialize PhysX extensions" << std::endl;
		return;
	}
	
#ifdef _DEBUG
	if (physics->getPvdConnectionManager()) {
		PxVisualDebuggerExt::createConnection(physics->getPvdConnectionManager(), "localhost", 5425, 10000);
		std::cout << "connected to PhysX visual debugger" << std::endl;
	}
	else {
		std::cout << "Unable to connect to the PhysX visual debugger" << std::endl;
	}
#endif
	
}

PhysxManager::~PhysxManager() {
	if (physics)
		physics->release();
	if (profileZoneManager)
		profileZoneManager->release();
	/*if (foundation)
		foundation->release(); */
}